@@ -1,12 +1,15 @@
 import os
 import importlib
+import json
 import logging
 from dataclasses import dataclass
 import sys
 
 from typing import Any, Dict, Optional, Tuple
 
-from ray.ray_constants import RAY_ADDRESS_ENVIRONMENT_VARIABLE
+from ray.ray_constants import (RAY_ADDRESS_ENVIRONMENT_VARIABLE,
+                               RAY_NAMESPACE_ENVIRONMENT_VARIABLE,
+                               RAY_RUNTIME_ENV_ENVIRONMENT_VARIABLE)
 from ray.job_config import JobConfig
 import ray.util.client_connect
 
@@ -55,12 +58,15 @@ class ClientContext:
 
 class ClientBuilder:
     """
-    Builder for a Ray Client connection.
+    Builder for a Ray Client connection. This class can be subclassed by
+    custom builder classes to modify connection behavior to include additional
+    features or altered semantics. One example is the ``_LocalClientBuilder``.
     """
 
     def __init__(self, address: Optional[str]) -> None:
         self.address = address
         self._job_config = JobConfig()
+        self._fill_defaults_from_env()
 
     def env(self, env: Dict[str, Any]) -> "ClientBuilder":
         """
@@ -74,12 +80,22 @@ class ClientBuilder:
         return self
 
     def namespace(self, namespace: str) -> "ClientBuilder":
+        """
+        Sets the namespace for the session.
+        Args:
+            namespace (str): Namespace to use.
+        """
         self._job_config.set_ray_namespace(namespace)
         return self
 
     def connect(self) -> ClientContext:
         """
         Begin a connection to the address passed in via ray.client(...).
+
+        Returns:
+            ClientInfo: Dataclass with information about the setting. This
+                includes the server's version of Python & Ray as well as the
+                dashboard_url.
         """
         client_info_dict = ray.util.client_connect.connect(
             self.address, job_config=self._job_config)
@@ -93,14 +109,87 @@ class ClientBuilder:
             protocol_version=client_info_dict["protocol_version"],
             _num_clients=client_info_dict["num_clients"])
 
+    def _fill_defaults_from_env(self):
+        # Check environment variables for default values
+        namespace_env_var = os.environ.get(RAY_NAMESPACE_ENVIRONMENT_VARIABLE)
+        if namespace_env_var and self._job_config.ray_namespace is None:
+            self.namespace(namespace_env_var)
+
+        runtime_env_var = os.environ.get(RAY_RUNTIME_ENV_ENVIRONMENT_VARIABLE)
+        if runtime_env_var and self._job_config.runtime_env is None:
+            self.env(json.loads(runtime_env_var))
+
+    def _init_args(self, **kwargs) -> "ClientBuilder":
+        """
+        When a client builder is constructed through ray.init, for example
+        `ray.init(ray://..., namespace=...)`, all of the
+        arguments passed into ray.init are passed again into this method.
+        Custom client builders can override this method to do their own
+        handling/validation of arguments.
+        """
+        # Use namespace and runtime_env from ray.init call
+        if kwargs.get("namespace") is not None:
+            self.namespace(kwargs["namespace"])
+            del kwargs["namespace"]
+        if kwargs.get("runtime_env") is not None:
+            self.env(kwargs["runtime_env"])
+            del kwargs["runtime_env"]
+        if not kwargs:
+            return self
+        unknown = ", ".join(kwargs)
+        raise RuntimeError(
+            f"Unexpected keyword argument(s) for Ray Client: {unknown}")
+
 
 class _LocalClientBuilder(ClientBuilder):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._init_args_dict = {}
+        self._internal_config = {}
+
+    def _init_args(self, **kwargs) -> "ClientBuilder":
+        for argument in kwargs:
+            if argument.startswith("_"):
+                raise RuntimeError(
+                    f"Unexpected keyword argument: {argument}. Unstable "
+                    "parameters using the local client should be passed "
+                    "through the `internal_config` dict, e.g. ray.init"
+                    "(local://..., internal_config={{argument: value}}")
+        if kwargs.get("job_config") is not None:
+            self._job_config = kwargs.pop("job_config")
+        if kwargs.get("namespace") is not None:
+            self.namespace(kwargs.pop("namespace"))
+        if kwargs.get("runtime_env") is not None:
+            self.env(kwargs.pop("runtime_env"))
+        self._internal_config = kwargs.pop("internal_config", {})
+
+        # Prefix internal_config names with underscores
+        arg_names = list(self._internal_config)
+        for name in arg_names:
+            if name.startswith("_"):
+                stripped_name = name.lstrip("_")
+                raise RuntimeError(
+                    f"Found internal_config argument "
+                    f"`{name}`. Arguments passed in internal_config should "
+                    "not include the underscore prefix. Use "
+                    f"`{stripped_name}` instead.")
+            self._internal_config["_" + name] = self._internal_config[name]
+            del self._internal_config[name]
+
+        self._fill_defaults_from_env()
+        self._init_args_dict = kwargs
+        return self
+
     def connect(self) -> ClientContext:
         """
-        Begin a connection to the address passed in via ray.client(...).
+        Begin a connection to the address passed in via ray.client(...) or
+        ray.init("local://...")
         """
         connection_dict = ray.init(
-            address=self.address, job_config=self._job_config)
+            address=self.address,
+            job_config=self._job_config,
+            **self._init_args_dict,
+            **self._internal_config)
         return ClientContext(
             dashboard_url=connection_dict["webui_url"],
             python_version="{}.{}.{}".format(
@@ -134,13 +223,14 @@ def _get_builder_from_address(address: Optional[str]) -> ClientBuilder:
             cluster_file = os.path.join(ray._private.utils.get_user_temp_dir(),
                                         "ray_current_cluster")
             with open(cluster_file, "r") as f:
-                address = f.read()
-                print(address)
+                address = f.read().strip()
         except FileNotFoundError:
             # `address` won't be set and we'll create a new cluster.
             pass
         return _LocalClientBuilder(address)
     module_string, inner_address = _split_address(address)
+    if module_string == "local":
+        return _LocalClientBuilder(inner_address or None)
     try:
         module = importlib.import_module(module_string)
     except Exception:
@@ -156,10 +246,12 @@ def client(address: Optional[str] = None) -> ClientBuilder:
     """
     Creates a ClientBuilder based on the provided address. The address can be
     of the following forms:
-    * None -> Connects to or creates a local cluster and connects to it.
-    * local -> Creates a new cluster locally and connects to it.
-    * IP:Port -> Connects to a Ray Client Server at the given address.
-    * module://inner_address -> load module.ClientBuilder & pass inner_address
+
+        * None: Connects to or creates a local cluster and connects to it.
+        * ``"local"``: Creates a new cluster locally and connects to it.
+        * ``"IP:Port"``: Connects to a Ray Client Server at the given address.
+        * ``"module://inner_address"``: load module.ClientBuilder & pass
+            inner_address
     """
     env_address = os.environ.get(RAY_ADDRESS_ENVIRONMENT_VARIABLE)
     if env_address and address is None:
